import re
import typing
from datetime import date, datetime, timedelta
from django.conf import settings
from django.utils import timezone

from greg.models import Identity


def camel_to_snake(s: str) -> str:
    """Turns `FooBar` into `foo_bar`."""
    return re.sub("([A-Z])", "_\\1", s).lower().lstrip("_")


def is_valid_id_number(input_digits: str) -> bool:
    """
    Checks whether the input represents a valid national ID number (fødselsnummer) or a valid D-number.
    """
    is_dnumber = int(input_digits[0:1]) >= 4
    return is_valid_norwegian_national_id_number(input_digits, is_dnumber)


def is_valid_norwegian_national_id_number(input_digits: str, is_dnumber: bool) -> bool:
    """
    Checks whether input_digits is a valid national ID number or D-number.

    It is based on the code found here:
    https://github.com/navikt/fnrvalidator/blob/master/src/validator.js
    """
    return (
        _check_correct_number_of_digits(input_digits)
        and _compute_checksum(input_digits)
        and _check_birthdate(input_digits, is_dnumber)
    )


def _check_correct_number_of_digits(input_digits: str) -> bool:
    return re.search("^\\d{11}$", input_digits) is not None


def _check_birthdate(digits: str, is_dnumber: bool) -> bool:
    if is_dnumber:
        # First number has been increased by 4, look here for an explanation of what the
        # numbers in the D-number mean:
        # https://www.skatteetaten.no/person/utenlandsk/norsk-identitetsnummer/d-nummer/
        digits = str((int(digits[0:1]) - 4)) + digits[1:]

    day = int(digits[0:2])
    month = int(digits[2:4])
    year = int(digits[4:6])

    # Year 0 should mean 2000, not 1900
    if year == 0:
        year = 2000

    try:
        # Try to create a date object, it will fail is the date is not valid
        date(year, month, day)
    except ValueError:
        # Not a valid date
        return False

    return True


def _compute_checksum(input_digits: str) -> bool:
    d = [int(s) for s in input_digits]
    k1 = 11 - (
        (
            3 * d[0]
            + 7 * d[1]
            + 6 * d[2]
            + 1 * d[3]
            + 8 * d[4]
            + 9 * d[5]
            + 4 * d[6]
            + 5 * d[7]
            + 2 * d[8]
        )
        % 11
    )

    k2 = 11 - (
        (
            5 * d[0]
            + 4 * d[1]
            + 3 * d[2]
            + 2 * d[3]
            + 7 * d[4]
            + 6 * d[5]
            + 5 * d[6]
            + 4 * d[7]
            + 3 * d[8]
            + 2 * k1
        )
        % 11
    )

    if k1 == 11:
        k1 = 0

    if k2 == 11:
        k2 = 0

    return k1 < 10 and k2 < 10 and k1 == d[9] and k2 == d[10]


def date_to_datetime_midnight(in_date: typing.Union[date, str]) -> datetime:
    """Convert a date or str object to a datetime object with timezone utc"""
    start_date = (
        datetime.strptime(in_date, "%Y-%M-%d") if isinstance(in_date, str) else in_date
    )
    start = datetime.combine(start_date, datetime.min.time(), tzinfo=timezone.utc)
    return start


def get_default_invitation_expire_date_from_now():
    return timezone.now() + timedelta(days=settings.INVITATION_DURATION)


def is_identity_duplicate(identity_type: Identity.IdentityType, value: str) -> bool:
    return Identity.objects.filter(type=identity_type).filter(value=value).exists()
